const express = require('express');
const axios = require('axios');
const fs = require('fs');
const path = require('path');
const server = express();
const config = require('./server-config.json');
const PORT = 3000;

const GOOGLE_SENTIMENT_URL = 'https://language.googleapis.com/v1/documents:analyzeSentiment';
const TWITTER_SEARCH_URL = 'https://api.twitter.com/1.1/search/tweets.json';

const FLAG_CACHE = process.argv.slice(2).some(arg => {
  return arg === '-c' || arg === '--cache';
});
if (FLAG_CACHE) {
  console.log('-- cache mode --');
}

server.use(express.static(path.join(__dirname, 'public')));
server.use(express.json());

function useCached(filepath, func) {
  if (! FLAG_CACHE || ! fs.existsSync(filepath)) {
    return func().then(text => {
      if (FLAG_CACHE && text !== null) {
        fs.writeFileSync(filepath, text);
      }
      return text;
    });
  } else {
    console.log(`-- use cache ${filepath} --`);
    const text = fs.readFileSync(filepath, 'utf8');
    return Promise.resolve(text);
  }
}

server.get('/tweets', (req, res) => {
  useCached(`./data/tweets_cache.json`, () => {
    const params = {
      q: req.query.keyword,
      count: 100
    };
    return axios.request({
      url: TWITTER_SEARCH_URL,
      method: 'get',
      params: params,
      headers: {
        'Authorization': 'Bearer ' + config.TWITTER_BEARER_TOKEN
      }
    }).then(response => {
      const results = response.data.statuses;
      const text = JSON.stringify(results);
      return text;
    });
  }).then(text => {
    res.header('Content-Type', 'application/json; charset=utf-8');
    res.send(text);
  }, error => {
    console.error(error.toJSON());
    res.status(500).send(error.toJSON());
  });
});

server.post('/sentiment', (req, res) => {
  useCached('./data/sentiment_cache.json', () => {
    return axios.request({
      url: GOOGLE_SENTIMENT_URL,
      method: 'post',
      params: {
        key: config.GOOGLE_API_KEY
      },
      headers: {
        'Content-Type': 'application/json'
      },
      data: {
        encodingType: 'UTF8',
        document: {
          type: 'PLAIN_TEXT',
          content: req.body.text
        }
      }
    }).then(response => {
      return JSON.stringify(response.data);
    }, error => {
      console.error(error.toJSON());
      res.status(500).send(error.toJSON());
    });
  }).then(text => {
    res.header('Content-Type', 'application/json; charset=utf-8');
    res.send(text);
  });
});

server.listen(PORT, () => {
  console.log(`Listening on port ${PORT}`);
});

